"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const redis = require("redis");
class GreenlockRedisChallenge {
    constructor(opts) {
        this._options = {
            prefix: 'letsencrypt-acme-challenge'
        };
        Object.assign(this._options, opts);
    }
    getOptions() {
        return this._options;
    }
    init() {
        return null;
    }
    set({ challenge }) {
        return this._exec(challenge, (id, client, done) => {
            client.set(id, challenge.keyAuthorization, (err) => done(err, null));
        });
    }
    get({ challenge }) {
        return this._exec(challenge, (id, client, done) => {
            client.get(id, (err, keyAuthorization) => done(err, keyAuthorization ? { keyAuthorization } : null));
        });
    }
    remove({ challenge }) {
        return this._exec(challenge, (id, client, done) => {
            client.del(id, (err) => done(err, null));
        });
    }
    _exec(challenge, fn) {
        const id = this._getId(challenge);
        const client = this._createClient();
        return new Promise((resolve, reject) => {
            fn(id, client, (err, ret) => {
                client.quit();
                if (err) {
                    reject(err);
                }
                else {
                    resolve(ret);
                }
            });
        });
    }
    _getId({ identifier: { value }, token }) {
        return `${this._options.prefix}:${value}:${token}`;
    }
    _createClient() {
        return redis.createClient(this._options.redisOptions);
    }
    static create(opts = {}) {
        return new GreenlockRedisChallenge(opts);
    }
}
exports.GreenlockRedisChallenge = GreenlockRedisChallenge;
